@@ -21,6 +21,9 @@ package org.codehaus.groovy.classgen;
 import groovy.lang.GroovyClassLoader;
 import groovy.lang.GroovyObject;
 import groovy.lang.MetaClass;
+import groovy.transform.Generated;
+import groovy.transform.Internal;
+import org.apache.groovy.ast.tools.ClassNodeUtils;
 import org.codehaus.groovy.GroovyBugError;
 import org.codehaus.groovy.ast.AnnotationNode;
 import org.codehaus.groovy.ast.ClassHelper;
@@ -52,7 +55,6 @@ import org.codehaus.groovy.ast.stmt.BlockStatement;
 import org.codehaus.groovy.ast.stmt.ExpressionStatement;
 import org.codehaus.groovy.ast.stmt.ReturnStatement;
 import org.codehaus.groovy.ast.stmt.Statement;
-import org.codehaus.groovy.ast.tools.ClassNodeUtils;
 import org.codehaus.groovy.ast.tools.GenericsUtils;
 import org.codehaus.groovy.ast.tools.PropertyNodeUtils;
 import org.codehaus.groovy.classgen.asm.BytecodeHelper;
@@ -87,7 +89,9 @@ import static java.lang.reflect.Modifier.isFinal;
 import static java.lang.reflect.Modifier.isPrivate;
 import static java.lang.reflect.Modifier.isPublic;
 import static java.lang.reflect.Modifier.isStatic;
-import static org.codehaus.groovy.ast.tools.GeneralUtils.makeDescriptorWithoutReturnType;
+import static org.apache.groovy.ast.tools.AnnotatedNodeUtils.markAsGenerated;
+import static org.apache.groovy.ast.tools.ExpressionUtils.transformInlineConstants;
+import static org.apache.groovy.ast.tools.MethodNodeUtils.methodDescriptorWithoutReturnType;
 import static org.codehaus.groovy.ast.tools.GenericsUtils.correctToGenericsSpec;
 import static org.codehaus.groovy.ast.tools.GenericsUtils.createGenericsSpec;
 
@@ -138,6 +142,9 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
             new Parameter(ClassHelper.METACLASS_TYPE, "mc")
     };
 
+    private static final Class GENERATED_ANNOTATION = Generated.class;
+    private static final Class INTERNAL_ANNOTATION = Internal.class;
+
     private ClassNode classNode;
     private MethodNode methodNode;
 
@@ -244,13 +251,44 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
         node.visitContents(this);
         checkForDuplicateMethods(node);
         addCovariantMethods(node);
+
+        checkFinalVariables(node);
+    }
+
+    private void checkFinalVariables(ClassNode node) {
+        FinalVariableAnalyzer analyzer = new FinalVariableAnalyzer(null, getFinalVariablesCallback());
+        analyzer.visitClass(node);
+
+    }
+
+    protected FinalVariableAnalyzer.VariableNotFinalCallback getFinalVariablesCallback() {
+        return new FinalVariableAnalyzer.VariableNotFinalCallback() {
+            @Override
+            public void variableNotFinal(Variable var, Expression bexp) {
+                if (var instanceof VariableExpression) {
+                    var = ((VariableExpression) var).getAccessedVariable();
+                }
+                if (var instanceof VariableExpression && isFinal(var.getModifiers())) {
+                    throw new RuntimeParserException("The variable [" + var.getName() + "] is declared final but is reassigned", bexp);
+                }
+                if (var instanceof Parameter && isFinal(var.getModifiers())) {
+                    throw new RuntimeParserException("The parameter [" + var.getName() + "] is declared final but is reassigned", bexp);
+                }
+            }
+
+            @Override
+            public void variableNotAlwaysInitialized(final VariableExpression var) {
+                if (Modifier.isFinal(var.getAccessedVariable().getModifiers()))
+                    throw new RuntimeParserException("The variable [" + var.getName() + "] may be uninitialized", var);
+            }
+        };
     }
 
     private static void checkForDuplicateMethods(ClassNode cn) {
         Set<String> descriptors = new HashSet<String>();
         for (MethodNode mn : cn.getMethods()) {
             if (mn.isSynthetic()) continue;
-            String mySig = makeDescriptorWithoutReturnType(mn);
+            String mySig = methodDescriptorWithoutReturnType(mn);
             if (descriptors.contains(mySig)) {
                 if (mn.isScriptBody() || mySig.equals(scriptBodySignatureWithoutReturnType(cn))) {
                     throw new RuntimeParserException("The method " + mn.getText() +
@@ -266,7 +304,7 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
 
     private static String scriptBodySignatureWithoutReturnType(ClassNode cn) {
         for (MethodNode mn : cn.getMethods()) {
-            if (mn.isScriptBody()) return makeDescriptorWithoutReturnType(mn);
+            if (mn.isScriptBody()) return methodDescriptorWithoutReturnType(mn);
         }
         return null;
     }
@@ -330,7 +368,7 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
                         if (BytecodeHelper.isClassLiteralPossible(node) || BytecodeHelper.isSameCompilationUnit(classNode, node)) {
                             BytecodeHelper.visitClassLiteral(mv, node);
                         } else {
-                            mv.visitMethodInsn(INVOKESTATIC, classInternalName, "$get$$class$" + classInternalName.replaceAll("\\/", "\\$"), "()Ljava/lang/Class;", false);
+                            mv.visitMethodInsn(INVOKESTATIC, classInternalName, "$get$$class$" + classInternalName.replaceAll("/", "\\$"), "()Ljava/lang/Class;", false);
                         }
                         Label l1 = new Label();
                         mv.visitJumpInsn(IF_ACMPEQ, l1);
@@ -369,9 +407,13 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
         if (!node.isDerivedFromGroovyObject()) node.addInterface(ClassHelper.make(GroovyObject.class));
         FieldNode metaClassField = getMetaClassField(node);
 
+        boolean shouldAnnotate = classNode.getModule().getContext() != null;
+        AnnotationNode generatedAnnotation = shouldAnnotate ? new AnnotationNode(ClassHelper.make(GENERATED_ANNOTATION)) : null;
+        AnnotationNode internalAnnotation = shouldAnnotate ? new AnnotationNode(ClassHelper.make(INTERNAL_ANNOTATION)) : null;
+
         if (!node.hasMethod("getMetaClass", Parameter.EMPTY_ARRAY)) {
             metaClassField = setMetaClassFieldIfNotExists(node, metaClassField);
-            addMethod(node, !isAbstract(node.getModifiers()),
+            MethodNode methodNode = addMethod(node, !isAbstract(node.getModifiers()),
                     "getMetaClass",
                     ACC_PUBLIC,
                     ClassHelper.METACLASS_TYPE,
@@ -380,7 +422,7 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
                     new BytecodeSequence(new BytecodeInstruction() {
                         public void visit(MethodVisitor mv) {
                             Label nullLabel = new Label();
-                            /**
+                            /*
                              *  the code is:
                              *  if (this.metaClass==null) {
                              *      this.metaClass = this.$getStaticMetaClass
@@ -410,6 +452,10 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
                         }
                     })
             );
+            if (shouldAnnotate) {
+                methodNode.addAnnotation(generatedAnnotation);
+                methodNode.addAnnotation(internalAnnotation);
+            }
         }
 
         Parameter[] parameters = new Parameter[]{new Parameter(ClassHelper.METACLASS_TYPE, "mc")};
@@ -424,7 +470,7 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
                 List list = new ArrayList();
                 list.add(new BytecodeInstruction() {
                     public void visit(MethodVisitor mv) {
-                        /**
+                        /*
                          * the code is (meta class is stored in 1):
                          * this.metaClass = <1>
                          */
@@ -438,12 +484,16 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
                 setMetaClassCode = new BytecodeSequence(list);
             }
 
-            addMethod(node, !isAbstract(node.getModifiers()),
+            MethodNode methodNode = addMethod(node, !isAbstract(node.getModifiers()),
                     "setMetaClass",
                     ACC_PUBLIC, ClassHelper.VOID_TYPE,
                     SET_METACLASS_PARAMS, ClassNode.EMPTY_ARRAY,
                     setMetaClassCode
             );
+            if (shouldAnnotate) {
+                methodNode.addAnnotation(generatedAnnotation);
+                methodNode.addAnnotation(internalAnnotation);
+            }
         }
 
         if (!node.hasMethod("invokeMethod", INVOKE_METHOD_PARAMS)) {
@@ -453,7 +503,7 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
             blockScope.putReferencedLocalVariable(vMethods);
             blockScope.putReferencedLocalVariable(vArguments);
 
-            addMethod(node, !isAbstract(node.getModifiers()),
+            MethodNode methodNode = addMethod(node, !isAbstract(node.getModifiers()),
                     "invokeMethod",
                     ACC_PUBLIC,
                     ClassHelper.OBJECT_TYPE, INVOKE_METHOD_PARAMS,
@@ -470,10 +520,14 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
                         }
                     })
             );
+            if (shouldAnnotate) {
+                methodNode.addAnnotation(generatedAnnotation);
+                methodNode.addAnnotation(internalAnnotation);
+            }
         }
 
         if (!node.hasMethod("getProperty", GET_PROPERTY_PARAMS)) {
-            addMethod(node, !isAbstract(node.getModifiers()),
+            MethodNode methodNode = addMethod(node, !isAbstract(node.getModifiers()),
                     "getProperty",
                     ACC_PUBLIC,
                     ClassHelper.OBJECT_TYPE,
@@ -490,10 +544,14 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
                         }
                     })
             );
+            if (shouldAnnotate) {
+                methodNode.addAnnotation(generatedAnnotation);
+                methodNode.addAnnotation(internalAnnotation);
+            }
         }
 
         if (!node.hasMethod("setProperty", SET_PROPERTY_PARAMS)) {
-            addMethod(node, !isAbstract(node.getModifiers()),
+            MethodNode methodNode = addMethod(node, !isAbstract(node.getModifiers()),
                     "setProperty",
                     ACC_PUBLIC,
                     ClassHelper.VOID_TYPE,
@@ -511,6 +569,10 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
                         }
                     })
             );
+            if (shouldAnnotate) {
+                methodNode.addAnnotation(generatedAnnotation);
+                methodNode.addAnnotation(internalAnnotation);
+            }
         }
     }
 
@@ -519,15 +581,21 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
      * call will either be made to ClassNode.addSyntheticMethod() or ClassNode.addMethod(). If a non-synthetic method
      * is to be added the ACC_SYNTHETIC modifier is removed if it has been accidentally supplied.
      */
-    protected void addMethod(ClassNode node, boolean shouldBeSynthetic, String name, int modifiers, ClassNode returnType, Parameter[] parameters,
-                             ClassNode[] exceptions, Statement code) {
+    protected MethodNode addMethod(ClassNode node, boolean shouldBeSynthetic, String name, int modifiers, ClassNode returnType, Parameter[] parameters,
+                                   ClassNode[] exceptions, Statement code) {
         if (shouldBeSynthetic) {
-            node.addSyntheticMethod(name, modifiers, returnType, parameters, exceptions, code);
+            return node.addSyntheticMethod(name, modifiers, returnType, parameters, exceptions, code);
         } else {
-            node.addMethod(name, modifiers & ~ACC_SYNTHETIC, returnType, parameters, exceptions, code);
+            return node.addMethod(name, modifiers & ~ACC_SYNTHETIC, returnType, parameters, exceptions, code);
         }
     }
 
+    // for binary compatibility only, don't use or override this
+    protected void addMethod$$bridge(ClassNode node, boolean shouldBeSynthetic, String name, int modifiers, ClassNode returnType, Parameter[] parameters,
+                                   ClassNode[] exceptions, Statement code) {
+        addMethod(node, shouldBeSynthetic, name, modifiers, returnType, parameters, exceptions, code);
+    }
+
     @Deprecated
     protected void addTimeStamp(ClassNode node) {
     }
@@ -671,19 +739,11 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
             getterModifiers = ~Modifier.FINAL & getterModifiers;
         }
         if (getterBlock != null) {
-            MethodNode getter =
-                    new MethodNode(getterName, getterModifiers, node.getType(), Parameter.EMPTY_ARRAY, ClassNode.EMPTY_ARRAY, getterBlock);
-            getter.setSynthetic(true);
-            addPropertyMethod(getter);
-            visitMethod(getter);
+            visitGetter(node, getterBlock, getterModifiers, getterName);
 
             if (ClassHelper.boolean_TYPE == node.getType() || ClassHelper.Boolean_TYPE == node.getType()) {
                 String secondGetterName = "is" + capitalize(name);
-                MethodNode secondGetter =
-                        new MethodNode(secondGetterName, getterModifiers, node.getType(), Parameter.EMPTY_ARRAY, ClassNode.EMPTY_ARRAY, getterBlock);
-                secondGetter.setSynthetic(true);
-                addPropertyMethod(secondGetter);
-                visitMethod(secondGetter);
+                visitGetter(node, getterBlock, getterModifiers, secondGetterName);
             }
         }
         if (setterBlock != null) {
@@ -696,8 +756,17 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
         }
     }
 
+    private void visitGetter(PropertyNode node, Statement getterBlock, int getterModifiers, String secondGetterName) {
+        MethodNode secondGetter =
+                new MethodNode(secondGetterName, getterModifiers, node.getType(), Parameter.EMPTY_ARRAY, ClassNode.EMPTY_ARRAY, getterBlock);
+        secondGetter.setSynthetic(true);
+        addPropertyMethod(secondGetter);
+        visitMethod(secondGetter);
+    }
+
     protected void addPropertyMethod(MethodNode method) {
         classNode.addMethod(method);
+        markAsGenerated(classNode, method);
         // GROOVY-4415 / GROOVY-4645: check that there's no abstract method which corresponds to this one
         List<MethodNode> abstractMethods = classNode.getAbstractMethods();
         if (abstractMethods == null) return;
@@ -821,7 +890,8 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
     }
 
     protected void addConstructor(Parameter[] newParams, ConstructorNode ctor, Statement code, ClassNode node) {
-        node.addConstructor(ctor.getModifiers(), newParams, ctor.getExceptions(), code);
+        ConstructorNode genConstructor = node.addConstructor(ctor.getModifiers(), newParams, ctor.getExceptions(), code);
+        markAsGenerated(node, genConstructor);
     }
 
     /**
@@ -873,22 +943,10 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
                         );
                         k++;
                     } else if (parameter.hasInitialExpression()) {
-                        newParams[index++] = parameter;
-                        arguments.addExpression(
-                                new CastExpression(
-                                        parameter.getType(),
-                                        new VariableExpression(parameter.getName())
-                                )
-                        );
+                        index = addExpression(newParams, arguments, index, parameter);
                         k++;
                     } else {
-                        newParams[index++] = parameter;
-                        arguments.addExpression(
-                                new CastExpression(
-                                        parameter.getType(),
-                                        new VariableExpression(parameter.getName())
-                                )
-                        );
+                        index = addExpression(newParams, arguments, index, parameter);
                     }
                 }
             }
@@ -896,12 +954,24 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
         }
 
         for (Parameter parameter : parameters) {
+            if (!parameter.hasInitialExpression()) continue; // GROOVY-8728 make idempotent
             // remove default expression and store it as node metadata
             parameter.putNodeMetaData(Verifier.INITIAL_EXPRESSION, parameter.getInitialExpression());
             parameter.setInitialExpression(null);
         }
     }
 
+    private int addExpression(Parameter[] newParams, ArgumentListExpression arguments, int index, Parameter parameter) {
+        newParams[index++] = parameter;
+        arguments.addExpression(
+                new CastExpression(
+                        parameter.getType(),
+                        new VariableExpression(parameter.getName())
+                )
+        );
+        return index;
+    }
+
     protected void addClosureCode(InnerClassNode node) {
         // add a new invoke
     }
@@ -1047,7 +1117,7 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
     }
 
     private static ConstructorCallExpression getFirstIfSpecialConstructorCall(Statement code) {
-        if (code == null || !(code instanceof ExpressionStatement)) return null;
+        if (!(code instanceof ExpressionStatement)) return null;
 
         Expression expression = ((ExpressionStatement) code).getExpression();
         if (!(expression instanceof ConstructorCallExpression)) return null;
@@ -1074,10 +1144,12 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
                 // GROOVY-3311: pre-defined constants added by groovy compiler for numbers/characters should be
                 // initialized first so that code dependent on it does not see their values as empty
                 Expression initialValueExpression = fieldNode.getInitialValueExpression();
-                if (initialValueExpression instanceof ConstantExpression) {
-                    ConstantExpression cexp = (ConstantExpression) initialValueExpression;
+                Expression transformed = transformInlineConstants(initialValueExpression, fieldNode.getType());
+                if (transformed instanceof ConstantExpression) {
+                    ConstantExpression cexp = (ConstantExpression) transformed;
                     cexp = transformToPrimitiveConstantIfPossible(cexp);
                     if (fieldNode.isFinal() && ClassHelper.isStaticConstantInitializerType(cexp.getType()) && cexp.getType().equals(fieldNode.getType())) {
+                        fieldNode.setInitialValueExpression(transformed);
                         return; // GROOVY-5150: primitive type constants will be initialized directly
                     }
                     staticList.add(0, statement);
@@ -1140,6 +1212,17 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
 
     }
 
+    public static Long getTimestampFromFieldName(String fieldName) {
+        if (fieldName.startsWith(__TIMESTAMP__)) {
+            try {
+                return Long.decode(fieldName.substring(__TIMESTAMP__.length()));
+            } catch (NumberFormatException e) {
+                return Long.MAX_VALUE;
+            }
+        }
+        return null;
+    }
+
     public static long getTimestamp(Class clazz) {
         if (clazz.getClassLoader() instanceof GroovyClassLoader.InnerLoader) {
             GroovyClassLoader.InnerLoader innerLoader = (GroovyClassLoader.InnerLoader) clazz.getClassLoader();
@@ -1149,13 +1232,9 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
         final Field[] fields = clazz.getFields();
         for (int i = 0; i != fields.length; ++i) {
             if (isStatic(fields[i].getModifiers())) {
-                final String name = fields[i].getName();
-                if (name.startsWith(__TIMESTAMP__)) {
-                    try {
-                        return Long.decode(name.substring(__TIMESTAMP__.length())).longValue();
-                    } catch (NumberFormatException e) {
-                        return Long.MAX_VALUE;
-                    }
+                Long timestamp = getTimestampFromFieldName(fields[i].getName());
+                if (timestamp != null) {
+                    return timestamp;
                 }
             }
         }
@@ -1167,9 +1246,9 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
         Map genericsSpec = new HashMap();
 
         // unimplemented abstract methods from interfaces
-        Map<String, MethodNode> abstractMethods = ClassNodeUtils.getDeclaredMethodMapsFromInterfaces(classNode);
+        Map<String, MethodNode> abstractMethods = ClassNodeUtils.getDeclaredMethodsFromInterfaces(classNode);
         Map<String, MethodNode> allInterfaceMethods = new HashMap<String, MethodNode>(abstractMethods);
-        ClassNodeUtils.addDeclaredMethodMapsFromSuperInterfaces(classNode, allInterfaceMethods);
+        ClassNodeUtils.addDeclaredMethodsFromAllInterfaces(classNode, allInterfaceMethods);
 
         List<MethodNode> declaredMethods = new ArrayList<MethodNode>(classNode.getMethods());
         // remove all static, private and package private methods
@@ -1215,11 +1294,7 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
             Map genericsSpec = createGenericsSpec(sn, oldGenericsSpec);
             List<MethodNode> classMethods = sn.getMethods();
             // original class causing bridge methods for methods in super class
-            for (Object declaredMethod : declaredMethods) {
-                MethodNode method = (MethodNode) declaredMethod;
-                if (method.isStatic()) continue;
-                storeMissingCovariantMethods(classMethods, method, methodsToAdd, genericsSpec, false);
-            }
+            storeMissingCovariantMethods(declaredMethods, methodsToAdd, genericsSpec, classMethods);
             // super class causing bridge methods for abstract methods in original class
             if (!abstractMethods.isEmpty()) {
                 for (Object classMethod : classMethods) {
@@ -1236,16 +1311,20 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
         for (ClassNode anInterface : interfaces) {
             List interfacesMethods = anInterface.getMethods();
             Map genericsSpec = createGenericsSpec(anInterface, oldGenericsSpec);
-            for (Object declaredMethod : declaredMethods) {
-                MethodNode method = (MethodNode) declaredMethod;
-                if (method.isStatic()) continue;
-                storeMissingCovariantMethods(interfacesMethods, method, methodsToAdd, genericsSpec, false);
-            }
+            storeMissingCovariantMethods(declaredMethods, methodsToAdd, genericsSpec, interfacesMethods);
             addCovariantMethods(anInterface, declaredMethods, abstractMethods, methodsToAdd, genericsSpec);
         }
 
     }
 
+    private void storeMissingCovariantMethods(List declaredMethods, Map methodsToAdd, Map genericsSpec, List<MethodNode> methodNodeList) {
+        for (Object declaredMethod : declaredMethods) {
+            MethodNode method = (MethodNode) declaredMethod;
+            if (method.isStatic()) continue;
+            storeMissingCovariantMethods(methodNodeList, method, methodsToAdd, genericsSpec, false);
+        }
+    }
+
     private MethodNode getCovariantImplementation(final MethodNode oldMethod, final MethodNode overridingMethod, Map genericsSpec, boolean ignoreError) {
         // method name
         if (!oldMethod.getName().equals(overridingMethod.getName())) return null;
