@@ -23,22 +23,23 @@ import org.gradle.api.internal.file.archive.compression.GzipArchiver;
 import org.gradle.api.resources.ResourceHandler;
 import org.gradle.api.resources.TextResourceFactory;
 import org.gradle.api.resources.internal.ReadableResourceInternal;
+import org.gradle.internal.resource.TextResourceLoader;
 
 public class DefaultResourceHandler implements ResourceHandler {
-    private final FileOperations fileOperations;
+    private final ResourceResolver resourceResolver;
     private final TextResourceFactory textResourceFactory;
 
-    public DefaultResourceHandler(FileOperations fileOperations, TemporaryFileProvider tempFileProvider) {
-        this.fileOperations = fileOperations;
-        textResourceFactory = new DefaultTextResourceFactory(fileOperations, tempFileProvider);
+    public DefaultResourceHandler(FileOperations fileOperations, ResourceResolver resourceResolver, TemporaryFileProvider tempFileProvider, TextResourceLoader textResourceLoader) {
+        this.resourceResolver = resourceResolver;
+        textResourceFactory = new DefaultTextResourceFactory(fileOperations, tempFileProvider, textResourceLoader);
     }
 
     public ReadableResourceInternal gzip(Object path) {
-        return new GzipArchiver(fileOperations.getFileResolver().resolveResource(path));
+        return new GzipArchiver(resourceResolver.resolveResource(path));
     }
 
     public ReadableResourceInternal bzip2(Object path) {
-        return new Bzip2Archiver(fileOperations.getFileResolver().resolveResource(path));
+        return new Bzip2Archiver(resourceResolver.resolveResource(path));
     }
 
     public TextResourceFactory getText() {
