@@ -27,19 +27,22 @@ import io.micronaut.core.naming.NameUtils;
 import io.micronaut.core.naming.conventions.StringConvention;
 import io.micronaut.core.reflect.ClassUtils;
 import io.micronaut.core.type.Argument;
-import io.micronaut.core.util.ArgumentUtils;
 import io.micronaut.core.util.CollectionUtils;
 import io.micronaut.core.util.StringUtils;
 import io.micronaut.core.value.MapPropertyResolver;
 import io.micronaut.core.value.PropertyResolver;
+import org.jetbrains.annotations.NotNull;
 import org.slf4j.Logger;
 
 import javax.annotation.Nonnull;
 import javax.annotation.Nullable;
 import java.util.*;
 import java.util.concurrent.ConcurrentHashMap;
+import java.util.function.Consumer;
+import java.util.function.Supplier;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
+import java.util.stream.Stream;
 
 /**
  * <p>A {@link PropertyResolver} that resolves from one or many {@link PropertySource} instances.</p>
@@ -50,9 +53,10 @@ import java.util.regex.Pattern;
 public class PropertySourcePropertyResolver implements PropertyResolver {
 
     private static final Logger LOG = ClassUtils.getLogger(PropertySourcePropertyResolver.class);
+    private static final Pattern DOT_PATTERN = Pattern.compile("\\.");
     private static final Pattern RANDOM_PATTERN = Pattern.compile("\\$\\{\\s?random\\.(\\S+?)\\}");
     private static final char[] DOT_DASH = new char[] {'.', '-'};
-
+    private static final Object NO_VALUE = new Object();
     protected final ConversionService<?> conversionService;
     protected final PropertyPlaceholderResolver propertyPlaceholderResolver;
     protected final Map<String, PropertySource> propertySources = new ConcurrentHashMap<>(10);
@@ -60,9 +64,10 @@ public class PropertySourcePropertyResolver implements PropertyResolver {
     // this allows optimization of searches by prefix
     @SuppressWarnings("MagicNumber")
     protected final Map<String, Object>[] catalog = new Map[58];
+    protected final Map<String, Object>[] rawCatalog = new Map[58];
     private final Random random = new Random();
     private final Map<String, Boolean> containsCache = new ConcurrentHashMap<>(20);
-    private final Map<String, Optional<?>> resolvedValueCache = new ConcurrentHashMap<>(20);
+    private final Map<String, Object> resolvedValueCache = new ConcurrentHashMap<>(20);
 
     /**
      * Creates a new, initially empty, {@link PropertySourcePropertyResolver} for the given {@link ConversionService}.
@@ -103,7 +108,6 @@ public class PropertySourcePropertyResolver implements PropertyResolver {
      */
     public PropertySourcePropertyResolver addPropertySource(@Nullable PropertySource propertySource) {
         if (propertySource != null) {
-            propertySources.put(propertySource.getName(), propertySource);
             processPropertySource(propertySource, propertySource.getConvention());
         }
         return this;
@@ -130,13 +134,17 @@ public class PropertySourcePropertyResolver implements PropertyResolver {
         } else {
             Boolean result = containsCache.get(name);
             if (result == null) {
-                Map<String, Object> entries = resolveEntriesForKey(name, false);
-                if (entries == null) {
-                    result = false;
-                } else {
-                    name = trimIndex(name);
-                    result = entries.containsKey(name) || entries.containsKey(normalizeName(name));
-                }
+                String finalName = trimIndex(name);
+                result = Stream.of(null, StringConvention.RAW).anyMatch(convention -> {
+                    Map<String, Object> entries = resolveEntriesForKey(finalName, false, convention);
+                    if (entries == null) {
+                        return false;
+                    } else {
+
+                        return entries.containsKey(finalName);
+                    }
+                });
+
                 containsCache.put(name, result);
             }
             return result;
@@ -148,31 +156,50 @@ public class PropertySourcePropertyResolver implements PropertyResolver {
         if (StringUtils.isEmpty(name)) {
             return false;
         } else {
-            Map<String, Object> entries = resolveEntriesForKey(name, false);
-            if (entries == null) {
-                return false;
-            } else {
-                name = trimIndex(name);
-                if (entries.containsKey(name) || entries.containsKey(normalizeName(name))) {
-                    return true;
+            String trimmedName = trimIndex(name);
+            return Stream.of(null, StringConvention.RAW).anyMatch(convention -> {
+                Map<String, Object> entries = resolveEntriesForKey(trimmedName, false, convention);
+                if (entries == null) {
+                    return false;
                 } else {
-                    String finalName = name + ".";
-                    return entries.keySet().stream().anyMatch(key ->
-                            key.startsWith(finalName)
-                    );
+
+                    if (entries.containsKey(trimmedName)) {
+                        return true;
+                    } else {
+                        String finalName = trimmedName + ".";
+                        return entries.keySet().stream().anyMatch(key ->
+                                key.startsWith(finalName)
+                        );
+                    }
                 }
-            }
+            });
+
         }
     }
 
     @Override
     public @Nonnull Map<String, Object> getProperties(String name, StringConvention keyFormat) {
-        if (keyFormat == null) {
-            keyFormat = StringConvention.RAW;
-        }
         if (!StringUtils.isEmpty(name)) {
-            Map<String, Object> entries = resolveEntriesForKey(name, false);
+            Map<String, Object> entries = resolveEntriesForKey(name, false, keyFormat);
             if (entries != null) {
+                if (keyFormat == null) {
+                    keyFormat = StringConvention.RAW;
+                }
+                return resolveSubMap(
+                        name,
+                        entries,
+                        ConversionContext.of(Map.class),
+                        keyFormat,
+                        MapFormat.MapTransformation.FLAT
+                );
+            } else {
+                entries = resolveEntriesForKey(name, false, null);
+                if (keyFormat == null) {
+                    keyFormat = StringConvention.RAW;
+                }
+                if (entries == null) {
+                    return Collections.emptyMap();
+                }
                 return resolveSubMap(
                         name,
                         entries,
@@ -190,26 +217,37 @@ public class PropertySourcePropertyResolver implements PropertyResolver {
         if (StringUtils.isEmpty(name)) {
             return Optional.empty();
         } else {
-            ArgumentUtils.requireNonNull("conversionContext", conversionContext);
+            Objects.requireNonNull(conversionContext, "Conversion context should not be null");
             Class<T> requiredType = conversionContext.getArgument().getType();
-            boolean cacheableType = requiredType == Boolean.class || requiredType == String.class;
-            String cacheName = name + '|' + requiredType.getSimpleName();
-            if (cacheableType && resolvedValueCache.containsKey(cacheName)) {
-                return (Optional<T>) resolvedValueCache.get(cacheName);
+            boolean cacheableType = ClassUtils.isJavaLangType(requiredType);
+            Object cached = cacheableType ? resolvedValueCache.get(cacheKey(name, requiredType)) : null;
+            if (cached != null) {
+                return cached == NO_VALUE ? Optional.empty() : Optional.of((T) cached);
             } else {
-                Map<String, Object> entries = resolveEntriesForKey(name, false);
+                Map<String, Object> entries = resolveEntriesForKey(name, false, null);
+                if (entries == null) {
+                    entries = resolveEntriesForKey(name, false, StringConvention.RAW);
+                }
                 if (entries != null) {
                     Object value = entries.get(name);
                     if (value == null) {
                         value = entries.get(normalizeName(name));
+                        if (value == null && name.indexOf('[') == -1) {
+                            // last chance lookup the raw value
+                            Map<String, Object> rawEntries = resolveEntriesForKey(name, false, StringConvention.RAW);
+                            value = rawEntries != null ? rawEntries.get(name) : null;
+                            if (value != null) {
+                                entries = rawEntries;
+                            }
+                        }
                     }
                     if (value == null) {
                         int i = name.indexOf('[');
                         if (i > -1 && name.endsWith("]")) {
                             String newKey = name.substring(0, i);
                             value = entries.get(newKey);
+                            String index = name.substring(i + 1, name.length() - 1);
                             if (value != null) {
-                                String index = name.substring(i + 1, name.length() - 1);
                                 if (StringUtils.isNotEmpty(index)) {
                                     if (value instanceof List) {
                                         try {
@@ -226,7 +264,6 @@ public class PropertySourcePropertyResolver implements PropertyResolver {
                                     }
                                 }
                             } else {
-                                String index = name.substring(i + 1, name.length() - 1);
                                 if (StringUtils.isNotEmpty(index)) {
                                     String subKey = newKey + '.' + index;
                                     value = entries.get(subKey);
@@ -247,13 +284,12 @@ public class PropertySourcePropertyResolver implements PropertyResolver {
                         }
 
                         if (cacheableType) {
-                            resolvedValueCache.put(cacheName, converted);
+                            resolvedValueCache.put(cacheKey(name, requiredType), converted.orElse((T) NO_VALUE));
                         }
                         return converted;
                     } else if (cacheableType) {
-                        Optional<?> e = Optional.empty();
-                        resolvedValueCache.put(cacheName, e);
-                        return (Optional<T>) e;
+                        resolvedValueCache.put(cacheKey(name, requiredType), NO_VALUE);
+                        return Optional.empty();
                     } else if (Properties.class.isAssignableFrom(requiredType)) {
                         Properties properties = resolveSubProperties(name, entries, conversionContext);
                         return Optional.of((T) properties);
@@ -281,13 +317,32 @@ public class PropertySourcePropertyResolver implements PropertyResolver {
         return Optional.empty();
     }
 
+    @NotNull
+    private <T> String cacheKey(@Nonnull String name, Class<T> requiredType) {
+        return name + '|' + requiredType.getSimpleName();
+    }
+
     /**
      * Returns a combined Map of all properties in the catalog.
      *
      * @return Map of all properties
+     * @deprecated Use {@link #getAllProperties(StringConvention, MapFormat.MapTransformation)} instead
      */
+    @Deprecated
     public Map<String, Object> getAllProperties() {
+        return getAllProperties(StringConvention.RAW, MapFormat.MapTransformation.NESTED);
+    }
+
+    /**
+     * Returns a combined Map of all properties in the catalog.
+     *
+     * @param keyConvention The map key convention
+     * @param transformation The map format
+     * @return Map of all properties
+     */
+    public Map<String, Object> getAllProperties(StringConvention keyConvention, MapFormat.MapTransformation transformation) {
         Map<String, Object> map = new HashMap<>();
+        boolean isNested = transformation == MapFormat.MapTransformation.NESTED;
 
         Arrays
             .stream(catalog)
@@ -295,12 +350,12 @@ public class PropertySourcePropertyResolver implements PropertyResolver {
             .map(Map::entrySet)
             .flatMap(Collection::stream)
             .forEach((Map.Entry<String, Object> entry) -> {
-                String k = entry.getKey();
+                String k = keyConvention.format(entry.getKey());
                 Object value = resolvePlaceHoldersIfNecessary(entry.getValue());
                 Map finalMap = map;
                 int index = k.indexOf('.');
-                if (index != -1) {
-                    String[] keys = k.split("\\.");
+                if (index != -1 && isNested) {
+                    String[] keys = DOT_PATTERN.split(k);
                     for (int i = 0; i < keys.length - 1; i++) {
                         if (!finalMap.containsKey(keys[i])) {
                             finalMap.put(keys[i], new HashMap<>());
@@ -329,8 +384,11 @@ public class PropertySourcePropertyResolver implements PropertyResolver {
         // special handling for maps for resolving sub keys
         Properties properties = new Properties();
         AnnotationMetadata annotationMetadata = conversionContext.getAnnotationMetadata();
-        StringConvention keyConvention = annotationMetadata.getValue(MapFormat.class, "keyFormat", StringConvention.class)
-                                                           .orElse(StringConvention.RAW);
+        StringConvention keyConvention = annotationMetadata.enumValue(MapFormat.class, "keyFormat", StringConvention.class)
+                                                           .orElse(null);
+        if (keyConvention == StringConvention.RAW) {
+            entries = resolveEntriesForKey(name, false, keyConvention);
+        }
         String prefix = name + '.';
         entries.entrySet().stream()
             .filter(map -> map.getKey().startsWith(prefix))
@@ -338,7 +396,7 @@ public class PropertySourcePropertyResolver implements PropertyResolver {
                 Object value = entry.getValue();
                 if (value != null) {
                     String key = entry.getKey().substring(prefix.length());
-                    key = keyConvention.format(key);
+                    key = keyConvention != null ? keyConvention.format(key) : key;
                     properties.put(key, resolvePlaceHoldersIfNecessary(value.toString()));
                 }
             });
@@ -355,8 +413,11 @@ public class PropertySourcePropertyResolver implements PropertyResolver {
     protected Map<String, Object> resolveSubMap(String name, Map<String, Object> entries, ArgumentConversionContext<?> conversionContext) {
         // special handling for maps for resolving sub keys
         AnnotationMetadata annotationMetadata = conversionContext.getAnnotationMetadata();
-        StringConvention keyConvention = annotationMetadata.getValue(MapFormat.class, "keyFormat", StringConvention.class).orElse(StringConvention.RAW);
-        MapFormat.MapTransformation transformation = annotationMetadata.getValue(
+        StringConvention keyConvention = annotationMetadata.enumValue(MapFormat.class, "keyFormat", StringConvention.class).orElse(null);
+        if (keyConvention == StringConvention.RAW) {
+            entries = resolveEntriesForKey(name, false, keyConvention);
+        }
+        MapFormat.MapTransformation transformation = annotationMetadata.enumValue(
                 MapFormat.class,
                 "transformation",
                 MapFormat.MapTransformation.class)
@@ -379,7 +440,7 @@ public class PropertySourcePropertyResolver implements PropertyResolver {
             String name,
             Map<String, Object> entries,
             ArgumentConversionContext<?> conversionContext,
-            StringConvention keyConvention,
+            @Nullable StringConvention keyConvention,
             MapFormat.MapTransformation transformation) {
         final Argument<?> valueType = conversionContext.getTypeVariable("V").orElse(Argument.OBJECT_ARGUMENT);
         Map<String, Object> subMap = new LinkedHashMap<>(entries.size());
@@ -392,7 +453,7 @@ public class PropertySourcePropertyResolver implements PropertyResolver {
                 Object value = resolvePlaceHoldersIfNecessary(entry.getValue());
 
                 if (transformation == MapFormat.MapTransformation.FLAT) {
-                    subMapKey = keyConvention.format(subMapKey);
+                    subMapKey = keyConvention != null ? keyConvention.format(subMapKey) : subMapKey;
                     value = conversionService.convert(value, valueType).orElse(null);
                     subMap.put(subMapKey, value);
                 } else {
@@ -442,46 +503,84 @@ public class PropertySourcePropertyResolver implements PropertyResolver {
                 List<String> resolvedProperties = resolvePropertiesForConvention(property, convention);
                 for (String resolvedProperty : resolvedProperties) {
                     int i = resolvedProperty.indexOf('[');
-                    if (i > -1 && resolvedProperty.endsWith("]")) {
-                        String index = resolvedProperty.substring(i + 1, resolvedProperty.length() - 1);
-                        if (StringUtils.isNotEmpty(index)) {
-                            resolvedProperty = resolvedProperty.substring(0, i);
-                            Map entries = resolveEntriesForKey(resolvedProperty, true);
-                            Object v = entries.get(resolvedProperty);
-                            if (StringUtils.isDigits(index)) {
-                                Integer number = Integer.valueOf(index);
-                                List list;
-                                if (v instanceof List) {
-                                    list = (List) v;
-                                } else {
-                                    list = new ArrayList(10);
-                                    entries.put(resolvedProperty, list);
-                                }
-                                fill(list, number, null);
-                                list.set(number, value);
-                            } else {
-                                Map map;
-                                if (v instanceof Map) {
-                                    map = (Map) v;
-                                } else {
-                                    map = new LinkedHashMap(10);
-                                    entries.put(resolvedProperty, map);
-                                }
-                                map.put(index, value);
-                            }
+                    if (i > -1) {
+                        String propertyName = resolvedProperty.substring(0, i);
+                        Map entries = resolveEntriesForKey(propertyName, true, null);
+                        if (entries != null) {
+                            processProperty(resolvedProperty.substring(i), val -> entries.put(propertyName, val), () -> entries.get(propertyName), value);
                         }
                     } else {
-
-                        Map entries = resolveEntriesForKey(resolvedProperty, true);
+                        Map entries = resolveEntriesForKey(resolvedProperty, true, null);
                         if (entries != null) {
                             entries.put(resolvedProperty, value);
                         }
                     }
                 }
+
+                final Map<String, Object> rawEntries = resolveEntriesForKey(property, true, StringConvention.RAW);
+                if (rawEntries != null) {
+                    rawEntries.put(property, value);
+                }
             }
         }
     }
 
+    private void processProperty(String property, Consumer<Object> containerSet, Supplier<Object> containerGet, Object actualValue) {
+        if (StringUtils.isEmpty(property)) {
+            containerSet.accept(actualValue);
+            return;
+        }
+        int i = property.indexOf('[');
+        int li = property.indexOf(']');
+        if (i == 0 && li > -1) {
+            String propertyIndex = property.substring(1, li);
+            String propertyRest = property.substring(li + 1);
+            Object container = containerGet.get();
+            if (StringUtils.isDigits(propertyIndex)) {
+                Integer number = Integer.valueOf(propertyIndex);
+                List list;
+                if (container instanceof List) {
+                    list = (List) container;
+                } else {
+                    list = new ArrayList(10);
+                    containerSet.accept(list);
+                }
+                fill(list, number, null);
+
+                processProperty(propertyRest, val -> list.set(number, val), () -> list.get(number), actualValue);
+            } else {
+                Map map;
+                if (container instanceof Map) {
+                    map = (Map) container;
+                } else {
+                    map = new LinkedHashMap(10);
+                    containerSet.accept(map);
+                }
+
+                processProperty(propertyRest, val -> map.put(propertyIndex, val), () -> map.get(propertyIndex), actualValue);
+            }
+        } else if (property.startsWith(".")) {
+            String propertyName;
+            String propertyRest;
+            if (i > -1) {
+                propertyName = property.substring(1, i);
+                propertyRest = property.substring(i);
+            } else {
+                propertyName = property.substring(1);
+                propertyRest = "";
+            }
+            Object v = containerGet.get();
+            Map map;
+            if (v instanceof Map) {
+                map = (Map) v;
+            } else {
+                map = new LinkedHashMap(10);
+                containerSet.accept(map);
+            }
+            processProperty(propertyRest, val -> map.put(propertyName, val), () -> map.get(propertyName), actualValue);
+        }
+    }
+
     private CharSequence processRandomExpressions(PropertySource.PropertyConvention convention, String property, CharSequence str) {
         if (convention != PropertySource.PropertyConvention.ENVIRONMENT_VARIABLE && str.toString().contains(propertyPlaceholderResolver.getPrefix())) {
             StringBuffer newValue = new StringBuffer();
@@ -512,7 +611,7 @@ public class PropertySourcePropertyResolver implements PropertyResolver {
                         randomValue = UUID.randomUUID().toString();
                         break;
                     case "uuid2":
-                        randomValue = UUID.randomUUID().toString().replaceAll("-", "");
+                        randomValue = UUID.randomUUID().toString().replace("-", "");
                         break;
                     default:
                         throw new ConfigurationException("Invalid random expression " + matcher.group(0) + " for property: " + property);
@@ -536,10 +635,22 @@ public class PropertySourcePropertyResolver implements PropertyResolver {
      */
     @SuppressWarnings("MagicNumber")
     protected Map<String, Object> resolveEntriesForKey(String name, boolean allowCreate) {
+        return resolveEntriesForKey(name, allowCreate, null);
+    }
+
+    /**
+     * @param name        The name
+     * @param allowCreate Whether allows creation
+     * @param convention The string convention
+     * @return The map with the resolved entries for the name
+     */
+    @SuppressWarnings("MagicNumber")
+    protected Map<String, Object> resolveEntriesForKey(String name, boolean allowCreate, @Nullable StringConvention convention) {
         Map<String, Object> entries = null;
         if (name.length() == 0) {
             return null;
         }
+        final Map<String, Object>[] catalog = convention == StringConvention.RAW ? this.rawCatalog : this.catalog;
         char firstChar = name.charAt(0);
         if (Character.isLetter(firstChar)) {
             int index = ((int) firstChar) - 65;
@@ -562,15 +673,16 @@ public class PropertySourcePropertyResolver implements PropertyResolver {
         resolvedValueCache.clear();
     }
 
-    private void processSubmapKey(Map<String, Object> map, String key, Object value, StringConvention keyConvention) {
+    private void processSubmapKey(Map<String, Object> map, String key, Object value, @Nullable StringConvention keyConvention) {
         int index = key.indexOf('.');
+        final boolean hasKeyConvention = keyConvention != null;
         if (index == -1) {
-            key = keyConvention.format(key);
+            key = hasKeyConvention ? keyConvention.format(key) : key;
             map.put(key, value);
         } else {
 
             String mapKey = key.substring(0, index);
-            mapKey = keyConvention.format(mapKey);
+            mapKey = hasKeyConvention ? keyConvention.format(mapKey) : mapKey;
             if (!map.containsKey(mapKey)) {
                 map.put(mapKey, new LinkedHashMap<>());
             }
@@ -617,60 +729,58 @@ public class PropertySourcePropertyResolver implements PropertyResolver {
     }
 
     private List<String> resolvePropertiesForConvention(String property, PropertySource.PropertyConvention convention) {
-        switch (convention) {
-            case ENVIRONMENT_VARIABLE:
-                property = property.toLowerCase(Locale.ENGLISH);
+        if (convention == PropertySource.PropertyConvention.ENVIRONMENT_VARIABLE) {
+            property = property.toLowerCase(Locale.ENGLISH);
 
-                List<Integer> separatorIndexList = new ArrayList<>();
-                char[] propertyArr = property.toCharArray();
-                for (int i = 0; i < propertyArr.length; i++) {
-                    if (propertyArr[i] == '_') {
-                        separatorIndexList.add(i);
-                    }
+            List<Integer> separatorIndexList = new ArrayList<>();
+            char[] propertyArr = property.toCharArray();
+            for (int i = 0; i < propertyArr.length; i++) {
+                if (propertyArr[i] == '_') {
+                    separatorIndexList.add(i);
+                }
+            }
+
+            if (!separatorIndexList.isEmpty()) {
+                //store the index in the array where each separator is
+                int[] separatorIndexes = separatorIndexList.stream().mapToInt(Integer::intValue).toArray();
+
+                int separatorCount = separatorIndexes.length;
+                //halves is used to determine when to flip the separator
+                int[] halves = new int[separatorCount];
+                //stores the separator per half
+                byte[] separator = new byte[separatorCount];
+                //the total number of permutations. 2 to the power of the number of separators
+                int permutations = (int) Math.pow(2, separatorCount);
+
+                //initialize the halves
+                //ex 4, 2, 1 for A_B_C_D
+                for (int i = 0; i < halves.length; i++) {
+                    int start = (i == 0) ? permutations : halves[i - 1];
+                    halves[i] = start / 2;
                 }
 
-                if (!separatorIndexList.isEmpty()) {
-                    //store the index in the array where each separator is
-                    int[] separatorIndexes = separatorIndexList.stream().mapToInt(Integer::intValue).toArray();
+                String[] properties = new String[permutations];
 
-                    int separatorCount = separatorIndexes.length;
-                    //halves is used to determine when to flip the separator
-                    int[] halves = new int[separatorCount];
-                    //stores the separator per half
-                    byte[] separator = new byte[separatorCount];
-                    //the total number of permutations. 2 to the power of the number of separators
-                    int permutations = (int) Math.pow(2, separatorCount);
-
-                    //initialize the halves
-                    //ex 4, 2, 1 for A_B_C_D
-                    for (int i = 0; i < halves.length; i++) {
-                        int start = (i == 0) ? permutations : halves[i - 1];
-                        halves[i] = start / 2;
-                    }
-
-                    String[] properties = new String[permutations];
-
-                    for (int i = 0; i < permutations; i++) {
-                        int round = i + 1;
-                        for (int s = 0; s < separatorCount; s++) {
-                            //mutate the array with the separator
-                            propertyArr[separatorIndexes[s]] = DOT_DASH[separator[s]];
-                            if (round % halves[s] == 0) {
-                                separator[s] ^= 1;
-                            }
+                for (int i = 0; i < permutations; i++) {
+                    int round = i + 1;
+                    for (int s = 0; s < separatorCount; s++) {
+                        //mutate the array with the separator
+                        propertyArr[separatorIndexes[s]] = DOT_DASH[separator[s]];
+                        if (round % halves[s] == 0) {
+                            separator[s] ^= 1;
                         }
-                        properties[i] = new String(propertyArr);
                     }
-
-                    return Arrays.asList(properties);
-                } else {
-                    return Collections.singletonList(property);
+                    properties[i] = new String(propertyArr);
                 }
-            default:
-                return Collections.singletonList(
-                        NameUtils.hyphenate(property, true)
-                );
+
+                return Arrays.asList(properties);
+            } else {
+                return Collections.singletonList(property);
+            }
         }
+        return Collections.singletonList(
+                NameUtils.hyphenate(property, true)
+        );
     }
 
     private String trimIndex(String name) {
