@@ -159,6 +159,12 @@ public interface AnnotationMetadataDelegate extends AnnotationMetadataProvider,
         return getAnnotationMetadata().doubleValue(annotation);
     }
 
+    @Nonnull
+    @Override
+    default Map<String, Object> getDefaultValues(@Nonnull String annotation) {
+        return getAnnotationMetadata().getDefaultValues(annotation);
+    }
+
     @Override
     default @Nonnull <T> Optional<T> getValue(@Nonnull String annotation, @Nonnull Argument<T> requiredType) {
         return getAnnotationMetadata().getValue(annotation, requiredType);
