@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2008-2014, http://www.snakeyaml.org
+ * Copyright (c) 2008, http://www.snakeyaml.org
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,6 +16,7 @@
 package org.yaml.snakeyaml.constructor;
 
 import java.lang.reflect.Array;
+import java.lang.reflect.Modifier;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.EnumMap;
@@ -26,11 +27,17 @@ import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.SortedMap;
+import java.util.SortedSet;
+import java.util.TreeMap;
+import java.util.TreeSet;
 
+import org.yaml.snakeyaml.TypeDescription;
 import org.yaml.snakeyaml.composer.Composer;
 import org.yaml.snakeyaml.composer.ComposerException;
 import org.yaml.snakeyaml.error.YAMLException;
 import org.yaml.snakeyaml.introspector.PropertyUtils;
+import org.yaml.snakeyaml.nodes.CollectionNode;
 import org.yaml.snakeyaml.nodes.MappingNode;
 import org.yaml.snakeyaml.nodes.Node;
 import org.yaml.snakeyaml.nodes.NodeId;
@@ -48,8 +55,8 @@ public abstract class BaseConstructor {
             NodeId.class);
     /**
      * It maps the (explicit or implicit) tag to the Construct implementation.
-     * It is used: <br/>
-     * 1) explicit tag - if present. <br/>
+     * It is used:
+     * 1) explicit tag - if present.
      * 2) implicit tag - when the runtime class of the instance is unknown (the
      * node has the Object.class)
      */
@@ -60,8 +67,8 @@ public abstract class BaseConstructor {
      */
     protected final Map<String, Construct> yamlMultiConstructors = new HashMap<String, Construct>();
 
-    private Composer composer;
-    private final Map<Node, Object> constructedObjects;
+    protected Composer composer;
+    final Map<Node, Object> constructedObjects;
     private final Set<Node> recursiveObjects;
     private final ArrayList<RecursiveTuple<Map<Object, Object>, RecursiveTuple<Object, Object>>> maps2fill;
     private final ArrayList<RecursiveTuple<Set<Object>, Object>> sets2fill;
@@ -69,14 +76,28 @@ public abstract class BaseConstructor {
     protected Tag rootTag;
     private PropertyUtils propertyUtils;
     private boolean explicitPropertyUtils;
+    private boolean allowDuplicateKeys = true;
+
+    private boolean wrappedToRootException = false;
+
+    protected final Map<Class<? extends Object>, TypeDescription> typeDefinitions;
+    protected final Map<Tag, Class<? extends Object>> typeTags;
 
     public BaseConstructor() {
         constructedObjects = new HashMap<Node, Object>();
         recursiveObjects = new HashSet<Node>();
         maps2fill = new ArrayList<RecursiveTuple<Map<Object, Object>, RecursiveTuple<Object, Object>>>();
         sets2fill = new ArrayList<RecursiveTuple<Set<Object>, Object>>();
+        typeDefinitions = new HashMap<Class<? extends Object>, TypeDescription>();
+        typeTags = new HashMap<Tag, Class<? extends Object>>();
+
         rootTag = null;
         explicitPropertyUtils = false;
+
+        typeDefinitions.put(SortedMap.class, new TypeDescription(SortedMap.class, Tag.OMAP,
+                TreeMap.class));
+        typeDefinitions.put(SortedSet.class, new TypeDescription(SortedSet.class, Tag.SET,
+                TreeSet.class));
     }
 
     public void setComposer(Composer composer) {
@@ -85,7 +106,7 @@ public abstract class BaseConstructor {
 
     /**
      * Check if more documents available
-     * 
+     *
      * @return true when there are more YAML documents in the stream
      */
     public boolean checkData() {
@@ -95,7 +116,7 @@ public abstract class BaseConstructor {
 
     /**
      * Construct and return the next document
-     * 
+     *
      * @return constructed instance
      */
     public Object getData() {
@@ -110,39 +131,48 @@ public abstract class BaseConstructor {
 
     /**
      * Ensure that the stream contains a single document and construct it
-     * 
+     *
+     * @param type the class of the instance being created
      * @return constructed instance
-     * @throws ComposerException
-     *             in case there are more documents in the stream
+     * @throws ComposerException in case there are more documents in the stream
      */
     public Object getSingleData(Class<?> type) {
         // Ensure that the stream contains a single document and construct it
         Node node = composer.getSingleNode();
-        if (node != null) {
+        if (node != null && !Tag.NULL.equals(node.getTag())) {
             if (Object.class != type) {
                 node.setTag(new Tag(type));
             } else if (rootTag != null) {
                 node.setTag(rootTag);
             }
             return constructDocument(node);
+        } else {
+            Construct construct = yamlConstructors.get(Tag.NULL);
+            return construct.construct(node);
         }
-        return null;
     }
 
     /**
      * Construct complete YAML document. Call the second step in case of
      * recursive structures. At the end cleans all the state.
-     * 
-     * @param node
-     *            root Node
+     *
+     * @param node root Node
      * @return Java instance
      */
-    private Object constructDocument(Node node) {
-        Object data = constructObject(node);
-        fillRecursive();
-        constructedObjects.clear();
-        recursiveObjects.clear();
-        return data;
+    protected final Object constructDocument(Node node) {
+        try {
+            Object data = constructObject(node);
+            fillRecursive();
+            constructedObjects.clear();
+            recursiveObjects.clear();
+            return data;
+        } catch (RuntimeException e) {
+            if (wrappedToRootException && !(e instanceof YAMLException)) {
+                throw new YAMLException(e);
+            } else {
+                throw e;
+            }
+        }
     }
 
     private void fillRecursive() {
@@ -164,22 +194,28 @@ public abstract class BaseConstructor {
     /**
      * Construct object from the specified Node. Return existing instance if the
      * node is already constructed.
-     * 
-     * @param node
-     *            Node to be constructed
+     *
+     * @param node Node to be constructed
      * @return Java instance
      */
     protected Object constructObject(Node node) {
         if (constructedObjects.containsKey(node)) {
             return constructedObjects.get(node);
         }
+        return constructObjectNoCheck(node);
+    }
+
+    protected Object constructObjectNoCheck(Node node) {
         if (recursiveObjects.contains(node)) {
             throw new ConstructorException(null, null, "found unconstructable recursive node",
                     node.getStartMark());
         }
         recursiveObjects.add(node);
         Construct constructor = getConstructor(node);
-        Object data = constructor.construct(node);
+        Object data = (constructedObjects.containsKey(node)) ? constructedObjects.get(node)
+                : constructor.construct(node);
+
+        finalizeConstruction(node, data);
         constructedObjects.put(node, data);
         recursiveObjects.remove(node);
         if (node.isTwoStepsConstruction()) {
@@ -192,10 +228,9 @@ public abstract class BaseConstructor {
      * Get the constructor to construct the Node. For implicit tags if the
      * runtime class is known a dedicated Construct implementation is used.
      * Otherwise the constructor is chosen by the tag.
-     * 
-     * @param node
-     *            Node to be constructed
-     * @return Construct implementation for the specified node
+     *
+     * @param node {@link Node} to construct an instance from
+     * @return {@link Construct} implementation for the specified node
      */
     protected Construct getConstructor(Node node) {
         if (node.useClassConstructor()) {
@@ -214,10 +249,11 @@ public abstract class BaseConstructor {
         }
     }
 
-    protected Object constructScalar(ScalarNode node) {
+    protected String constructScalar(ScalarNode node) {
         return node.getValue();
     }
 
+    // >>>> DEFAULTS >>>>
     protected List<Object> createDefaultList(int initSize) {
         return new ArrayList<Object>(initSize);
     }
@@ -226,44 +262,109 @@ public abstract class BaseConstructor {
         return new LinkedHashSet<Object>(initSize);
     }
 
+    protected Map<Object, Object> createDefaultMap(int initSize) {
+        // respect order from YAML document
+        return new LinkedHashMap<Object, Object>(initSize);
+    }
+
     protected Object createArray(Class<?> type, int size) {
         return Array.newInstance(type.getComponentType(), size);
     }
 
-    @SuppressWarnings("unchecked")
-    protected List<? extends Object> constructSequence(SequenceNode node) {
-        List<Object> result;
-        if (List.class.isAssignableFrom(node.getType()) && !node.getType().isInterface()) {
-            // the root class may be defined (Vector for instance)
-            try {
-                result = (List<Object>) node.getType().newInstance();
-            } catch (Exception e) {
-                throw new YAMLException(e);
-            }
-        } else {
-            result = createDefaultList(node.getValue().size());
-        }
-        constructSequenceStep2(node, result);
-        return result;
+    // <<<< DEFAULTS <<<<
 
+    protected Object finalizeConstruction(Node node, Object data) {
+        final Class<? extends Object> type = node.getType();
+        if (typeDefinitions.containsKey(type)) {
+            return typeDefinitions.get(type).finalizeConstruction(data);
+        }
+        return data;
+    }
+
+    // >>>> NEW instance
+    protected Object newInstance(Node node) {
+        try {
+            return newInstance(Object.class, node);
+        } catch (InstantiationException e) {
+            throw new YAMLException(e);
+        }
+    }
+
+    final protected Object newInstance(Class<?> ancestor, Node node) throws InstantiationException {
+        return newInstance(ancestor, node, true);
+    }
+
+    protected Object newInstance(Class<?> ancestor, Node node, boolean tryDefault)
+            throws InstantiationException {
+        final Class<? extends Object> type = node.getType();
+        if (typeDefinitions.containsKey(type)) {
+            TypeDescription td = typeDefinitions.get(type);
+            final Object instance = td.newInstance(node);
+            if (instance != null) {
+                return instance;
+            }
+        }
+        if (tryDefault) {
+            /*
+             * Removed <code> have InstantiationException in case of abstract
+             * type
+             */
+            if (ancestor.isAssignableFrom(type) && !Modifier.isAbstract(type.getModifiers())) {
+                try {
+                    java.lang.reflect.Constructor<?> c = type.getDeclaredConstructor();
+                    c.setAccessible(true);
+                    return c.newInstance();
+                } catch (NoSuchMethodException e) {
+                    throw new InstantiationException("NoSuchMethodException:"
+                            + e.getLocalizedMessage());
+                } catch (Exception e) {
+                    throw new YAMLException(e);
+                }
+            }
+        }
+        throw new InstantiationException();
     }
 
     @SuppressWarnings("unchecked")
-    protected Set<? extends Object> constructSet(SequenceNode node) {
-        Set<Object> result;
-        if (!node.getType().isInterface()) {
-            // the root class may be defined
-            try {
-                result = (Set<Object>) node.getType().newInstance();
-            } catch (Exception e) {
-                throw new YAMLException(e);
-            }
-        } else {
-            result = createDefaultSet(node.getValue().size());
+    protected Set<Object> newSet(CollectionNode<?> node) {
+        try {
+            return (Set<Object>) newInstance(Set.class, node);
+        } catch (InstantiationException e) {
+            return createDefaultSet(node.getValue().size());
         }
+    }
+
+    @SuppressWarnings("unchecked")
+    protected List<Object> newList(SequenceNode node) {
+        try {
+            return (List<Object>) newInstance(List.class, node);
+        } catch (InstantiationException e) {
+            return createDefaultList(node.getValue().size());
+        }
+    }
+
+    @SuppressWarnings("unchecked")
+    protected Map<Object, Object> newMap(MappingNode node) {
+        try {
+            return (Map<Object, Object>) newInstance(Map.class, node);
+        } catch (InstantiationException e) {
+            return createDefaultMap(node.getValue().size());
+        }
+    }
+
+    // <<<< NEW instance
+
+    // >>>> Construct => NEW, 2ndStep(filling)
+    protected List<? extends Object> constructSequence(SequenceNode node) {
+        List<Object> result = newList(node);
         constructSequenceStep2(node, result);
         return result;
+    }
 
+    protected Set<? extends Object> constructSet(SequenceNode node) {
+        Set<Object> result = newSet(node);
+        constructSequenceStep2(node, result);
+        return result;
     }
 
     protected Object constructArray(SequenceNode node) {
@@ -282,18 +383,19 @@ public abstract class BaseConstructor {
         int index = 0;
         for (Node child : node.getValue()) {
             // Handle multi-dimensional arrays...
-            if ( child.getType() == Object.class ) {
+            if (child.getType() == Object.class) {
                 child.setType(componentType);
             }
-            
+
             final Object value = constructObject(child);
 
             if (componentType.isPrimitive()) {
                 // Null values are disallowed for primitives
-                if ( value == null ) {
-                    throw new NullPointerException ( "Unable to construct element value for " + child );
+                if (value == null) {
+                    throw new NullPointerException(
+                            "Unable to construct element value for " + child);
                 }
-                
+
                 // Primitive arrays require quite a lot of work.
                 if (byte.class.equals(componentType)) {
                     Array.setByte(array, index, ((Number) value).byteValue());
@@ -333,30 +435,20 @@ public abstract class BaseConstructor {
         return array;
     }
 
-    protected Map<Object, Object> createDefaultMap() {
-        // respect order from YAML document
-        return new LinkedHashMap<Object, Object>();
-    }
-
-    protected Set<Object> createDefaultSet() {
-        // respect order from YAML document
-        return new LinkedHashSet<Object>();
-    }
-
     protected Set<Object> constructSet(MappingNode node) {
-        Set<Object> set = createDefaultSet();
+        final Set<Object> set = newSet(node);
         constructSet2ndStep(node, set);
         return set;
     }
 
     protected Map<Object, Object> constructMapping(MappingNode node) {
-        Map<Object, Object> mapping = createDefaultMap();
+        final Map<Object, Object> mapping = newMap(node);
         constructMapping2ndStep(node, mapping);
         return mapping;
     }
 
     protected void constructMapping2ndStep(MappingNode node, Map<Object, Object> mapping) {
-        List<NodeTuple> nodeValue = (List<NodeTuple>) node.getValue();
+        List<NodeTuple> nodeValue = node.getValue();
         for (NodeTuple tuple : nodeValue) {
             Node keyNode = tuple.getKeyNode();
             Node valueNode = tuple.getValueNode();
@@ -366,29 +458,31 @@ public abstract class BaseConstructor {
                     key.hashCode();// check circular dependencies
                 } catch (Exception e) {
                     throw new ConstructorException("while constructing a mapping",
-                            node.getStartMark(), "found unacceptable key " + key, tuple
-                                    .getKeyNode().getStartMark(), e);
+                            node.getStartMark(), "found unacceptable key " + key,
+                            tuple.getKeyNode().getStartMark(), e);
                 }
             }
             Object value = constructObject(valueNode);
             if (keyNode.isTwoStepsConstruction()) {
-                /*
-                 * if keyObject is created it 2 steps we should postpone putting
-                 * it in map because it may have different hash after
-                 * initialization compared to clean just created one. And map of
-                 * course does not observe key hashCode changes.
-                 */
-                maps2fill.add(0,
-                        new RecursiveTuple<Map<Object, Object>, RecursiveTuple<Object, Object>>(
-                                mapping, new RecursiveTuple<Object, Object>(key, value)));
+                postponeMapFilling(mapping, key, value);
             } else {
                 mapping.put(key, value);
             }
         }
     }
 
+    /*
+     * if keyObject is created it 2 steps we should postpone putting
+     * it in map because it may have different hash after
+     * initialization compared to clean just created one. And map of
+     * course does not observe key hashCode changes.
+     */
+    protected void postponeMapFilling(Map<Object, Object> mapping, Object key, Object value) {
+        maps2fill.add(0, new RecursiveTuple(mapping, new RecursiveTuple(key, value)));
+    }
+
     protected void constructSet2ndStep(MappingNode node, Set<Object> set) {
-        List<NodeTuple> nodeValue = (List<NodeTuple>) node.getValue();
+        List<NodeTuple> nodeValue = node.getValue();
         for (NodeTuple tuple : nodeValue) {
             Node keyNode = tuple.getKeyNode();
             Object key = constructObject(keyNode);
@@ -401,19 +495,25 @@ public abstract class BaseConstructor {
                 }
             }
             if (keyNode.isTwoStepsConstruction()) {
-                /*
-                 * if keyObject is created it 2 steps we should postpone putting
-                 * it into the set because it may have different hash after
-                 * initialization compared to clean just created one. And set of
-                 * course does not observe value hashCode changes.
-                 */
-                sets2fill.add(0, new RecursiveTuple<Set<Object>, Object>(set, key));
+                postponeSetFilling(set, key);
             } else {
                 set.add(key);
             }
         }
     }
 
+    /*
+     * if keyObject is created it 2 steps we should postpone putting
+     * it into the set because it may have different hash after
+     * initialization compared to clean just created one. And set of
+     * course does not observe value hashCode changes.
+     */
+    protected void postponeSetFilling(Set<Object> set, Object key) {
+        sets2fill.add(0, new RecursiveTuple<Set<Object>, Object>(set, key));
+    }
+
+    // <<<< Costruct => NEW, 2ndStep(filling)
+
     // TODO protected List<Object[]> constructPairs(MappingNode node) {
     // List<Object[]> pairs = new LinkedList<Object[]>();
     // List<Node[]> nodeValue = (List<Node[]>) node.getValue();
@@ -429,6 +529,10 @@ public abstract class BaseConstructor {
     public void setPropertyUtils(PropertyUtils propertyUtils) {
         this.propertyUtils = propertyUtils;
         explicitPropertyUtils = true;
+        Collection<TypeDescription> tds = typeDefinitions.values();
+        for (TypeDescription typeDescription : tds) {
+            typeDescription.setPropertyUtils(propertyUtils);
+        }
     }
 
     public final PropertyUtils getPropertyUtils() {
@@ -438,6 +542,25 @@ public abstract class BaseConstructor {
         return propertyUtils;
     }
 
+    /**
+     * Make YAML aware how to parse a custom Class. If there is no root Class
+     * assigned in constructor then the 'root' property of this definition is
+     * respected.
+     *
+     * @param definition to be added to the Constructor
+     * @return the previous value associated with <tt>definition</tt>, or
+     * <tt>null</tt> if there was no mapping for <tt>definition</tt>.
+     */
+    public TypeDescription addTypeDescription(TypeDescription definition) {
+        if (definition == null) {
+            throw new NullPointerException("TypeDescription is required.");
+        }
+        Tag tag = definition.getTag();
+        typeTags.put(tag, definition.getType());
+        definition.setPropertyUtils(getPropertyUtils());
+        return typeDefinitions.put(definition.getType(), definition);
+    }
+
     private static class RecursiveTuple<T, K> {
         private final T _1;
         private final K _2;
@@ -459,4 +582,20 @@ public abstract class BaseConstructor {
     public final boolean isExplicitPropertyUtils() {
         return explicitPropertyUtils;
     }
+
+    public boolean isAllowDuplicateKeys() {
+        return allowDuplicateKeys;
+    }
+
+    public void setAllowDuplicateKeys(boolean allowDuplicateKeys) {
+        this.allowDuplicateKeys = allowDuplicateKeys;
+    }
+
+    public boolean isWrappedToRootException() {
+        return wrappedToRootException;
+    }
+
+    public void setWrappedToRootException(boolean wrappedToRootException) {
+        this.wrappedToRootException = wrappedToRootException;
+    }
 }
